"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.handler = exports.builder = exports.desc = exports.command = void 0;

var _graphql = require("graphql");

var _papaparse = _interopRequireDefault(require("papaparse"));

var _fsExtra = _interopRequireDefault(require("fs-extra"));

var _path = _interopRequireDefault(require("path"));

var _chalk = _interopRequireDefault(require("chalk"));

var _jsYaml = _interopRequireDefault(require("js-yaml"));

var _util = require("./util");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function asyncGeneratorStep(gen, resolve, reject, _next, _throw, key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { Promise.resolve(value).then(_next, _throw); } }

function _asyncToGenerator(fn) { return function () { var self = this, args = arguments; return new Promise(function (resolve, reject) { var gen = fn.apply(self, args); function _next(value) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "next", value); } function _throw(err) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "throw", err); } _next(undefined); }); }; }

// Plugin boilerplate
const command = 'maddsvc [name] [--id] [--desc] [--source] [--manifest] [--project] [--endpoint]';
exports.command = command;
const desc = 'Add or update a Maana Q service from source or manifest';
exports.desc = desc;
const builder = {
  name: {
    description: 'Name of the service (required for source, ignored for manifest)'
  },
  id: {
    alias: 'i',
    description: 'Optional unique identity of the service'
  },
  desc: {
    alias: 'd',
    description: 'Optional brief description of the service'
  },
  source: {
    alias: 's',
    description: "Model definition (GraphQL SDL) describing a fully 'managed' graph"
  },
  manifest: {
    alias: 'm',
    description: 'Manifest (JSON) describing a managed or unmanaged service'
  },
  endpoint: {
    alias: 'e',
    description: 'Optional endpoint to use'
  }
};
exports.builder = builder;

const handler = /*#__PURE__*/function () {
  var _ref = _asyncToGenerator(function* (context, argv) {
    // console.log("context", context);
    // console.log("argv", argv);
    const config = yield context.getProjectConfig(); // console.log("config", config);

    const configPath = config.configPath;
    const extensions = config.extensions || {}; // console.log("extensions", extensions);

    const options = extensions.maana || {}; // console.log("options", options);

    const endpoint = (0, _util.getEndpoint)(config, argv);
    if (!endpoint) return;
    const client = endpoint.getClient();
    let query;
    let variables;
    let manifest;

    if (argv.manifest) {
      const filePath = _path.default.parse(argv.manifest);

      const content = _fsExtra.default.readFileSync(argv.manifest, 'utf-8');

      if (filePath.ext === '.yml' || filePath.ext === '.yaml') {
        manifest = _jsYaml.default.safeLoad(content);
      } else if (filePath.ext === '.json') {
        manifest = JSON.parse(content);
      } else {
        console.log(_chalk.default.red(`✘ Unsupported manifest file type: ${filePath.ext}`));
        return;
      }
    } else if (argv.source) {
      manifest = {
        id: argv.id,
        name: argv.name,
        description: argv.desc,
        source: argv.source
      };
    } else {
      console.log(_chalk.default.red(`✘ Must specify either a service source or manifest`));
      return;
    }

    console.log('manifest', manifest);
    const schema = (0, _util.readFile)(manifest.source);
    variables = {
      input: {
        id: manifest.id,
        name: manifest.name,
        description: manifest.description,
        schema
      }
    };
    query = `
    mutation addServiceSource($input: AddServiceSourceInput!) {
      addServiceSource(input: $input)
    }
  `;
    context.spinner.start(_chalk.default.yellow(`Adding service`));
    const result = yield client.request(query, variables);

    if (result['errors']) {
      context.spinner.fail(_chalk.default.red(`Call failed! ${JSON.stringify(result['errors'])}`));
    } else {
      context.spinner.succeed(_chalk.default.green(`Call succeeded: ${_chalk.default.yellow((0, _util.ellipse)(JSON.stringify(result)))}`));
    }
  });

  return function handler(_x, _x2) {
    return _ref.apply(this, arguments);
  };
}();

exports.handler = handler;