"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.handler = exports.builder = exports.describe = exports.command = exports.defaultBoilerplates = void 0;

var _path = _interopRequireDefault(require("path"));

var _admZip = _interopRequireDefault(require("adm-zip"));

var _chalk = _interopRequireDefault(require("chalk"));

var _commandExists = _interopRequireDefault(require("command-exists"));

var _fs = _interopRequireDefault(require("fs"));

var _parseGithubUrl = _interopRequireDefault(require("parse-github-url"));

var _lodash = require("lodash");

var _request = _interopRequireDefault(require("request"));

var _rimraf = _interopRequireDefault(require("rimraf"));

var _crossSpawn = require("cross-spawn");

var _tmp = _interopRequireDefault(require("tmp"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function asyncGeneratorStep(gen, resolve, reject, _next, _throw, key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { Promise.resolve(value).then(_next, _throw); } }

function _asyncToGenerator(fn) { return function () { var self = this, args = arguments; return new Promise(function (resolve, reject) { var gen = fn.apply(self, args); function _next(value) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "next", value); } function _throw(err) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "throw", err); } _next(undefined); }); }; }

// Project boilerplates
const defaultBoilerplates = [// Services
// --------
{
  name: 'service-dotnet-csharp',
  description: 'Microservice: C#',
  repo: `https://github.com/maana-io/q-template-service-csharp/tree/master`,
  installCommand: ''
}, {
  name: 'service-node-js',
  description: 'Microservice: JavaScript',
  repo: `https://github.com/maana-io/q-template-service-javascript/tree/master`,
  installCommand: 'npm install'
}, {
  name: 'service-node-ts',
  description: 'Microservice: TypeScript',
  repo: `https://github.com/maana-io/q-template-service-typescript/tree/master`,
  installCommand: 'yarn install'
}, {
  name: 'service-node-ts-typegraphql',
  description: 'Microservice: TypeScript (TypeGraphQL)',
  repo: `https://github.com/maana-io/q-template-service-typescript-typegraphql/tree/master`,
  installCommand: 'yarn install'
}, {
  name: 'service-python-ariadne',
  description: 'Microservice: Python',
  repo: `https://github.com/maana-io/q-template-service-python-ariadne/tree/master`,
  installCommand: ''
}, {
  name: 'service-jvm-scala',
  description: 'Microservice: Scala',
  repo: `https://github.com/maana-io/q-template-service-scala/tree/master`
}, // Assistants
// ----------
{
  name: 'assistant-react-js',
  description: 'Assistant: React (JavaScript)',
  repo: `https://github.com/maana-io/q-template-assistant-react-javascript/tree/v3.2.3`,
  installCommand: 'yarn install'
}, {
  name: 'assistant-react-ts',
  description: 'Assistant: React (TypeScript)',
  repo: `https://github.com/maana-io/q-template-assistant-react-typescript/tree/v3.2.3`,
  installCommand: 'yarn install'
}, {
  name: 'assistant-react-advanced-js',
  description: 'Assistant: React Advanced (JavaScript)',
  repo: `https://github.com/maana-io/q-template-assistant-react-advanced-javascript/tree/v3.2.3`,
  installCommand: 'yarn install'
}, // Apps
// ----
{
  name: 'app-react-js',
  description: 'Application: React (JavaScript)',
  repo: `https://github.com/maana-io/q-template-app-react-javascript/tree/master`,
  installCommand: 'yarn install'
}]; // Plugin boilerplate

exports.defaultBoilerplates = defaultBoilerplates;
const command = 'mcreate [directory]';
exports.command = command;
const describe = 'Bootstrap a new Maana Knowledge Microservice, Bot, Assistant, or Knowledge Application';
exports.describe = describe;
const builder = {
  boilerplate: {
    alias: 'b',
    describe: 'Full URL or repo shorthand (e.g. `owner/repo`) to boilerplate GitHub repository',
    type: 'string'
  },
  'no-install': {
    describe: `Don't install project dependencies`,
    type: 'boolean',
    default: false
  }
}; //
// Internal helpers
//

exports.builder = builder;

const getZipInfo = function (boilerplate) {
  let baseUrl = boilerplate;
  let branch = 'master';
  let subDir = '';
  const branchMatches = boilerplate.match(/^(.*)\/tree\/([a-zA-Z-_0-9.]*)\/?(.*)$/);

  if (branchMatches) {
    baseUrl = branchMatches[1];
    branch = branchMatches[2];
    subDir = branchMatches[3];
  }

  if (subDir === undefined) {
    subDir = '';
  }

  if (!subDir.startsWith('/')) {
    subDir = '/' + subDir;
  }

  if (!subDir.endsWith('/')) {
    subDir = subDir + '/';
  } // For tag names that are formatted like v3.2.3, the zip folder name does not
  // contain the v at the beginning, so it needs to be removed..


  let zipFolderName = branch;

  if (/^v\d+\.\d+\.\d+$/.test(zipFolderName)) {
    zipFolderName = zipFolderName.slice(1);
  }

  const nameMatches = baseUrl.match(/github\.com\/(.*)\/(.*)$/);
  if (!nameMatches) return;
  const repoName = nameMatches[2];
  const url = `${baseUrl}/archive/${branch}.zip`;
  const path = `${repoName}-${zipFolderName}${subDir}`;
  return {
    url,
    path
  };
};

const getGitHubUrl = function (boilerplate) {
  const details = (0, _parseGithubUrl.default)(boilerplate);

  if (details.host && details.owner && details.repo) {
    const branch = details.branch ? `/tree/${details.branch}` : '';
    return `https://${details.host}/${details.repo}${branch}`;
  }
};

const shell = function (command) {
  return new Promise(function (resolve, reject) {
    const commandParts = command.split(' ');
    const cmd = (0, _crossSpawn.spawn)(commandParts[0], commandParts.slice(1), {
      cwd: process.cwd(),
      detached: false,
      stdio: 'inherit'
    });
    cmd.on('error', reject);
    cmd.on('close', resolve);
  });
}; //
// Exported functions
//


const handler = /*#__PURE__*/function () {
  var _ref = _asyncToGenerator(function* (context, argv) {
    let {
      boilerplate,
      directory,
      noInstall
    } = argv;

    if (directory && directory.match(/[A-Z]/)) {
      console.log(`Project/directory name cannot contain uppercase letters: ${directory}`);
      directory = undefined;
    }

    if (!directory) {
      const {
        newDir
      } = yield context.prompt({
        type: 'input',
        name: 'newDir',
        default: '.',
        message: 'Directory for new Maana project',
        validate: function (dir) {
          if (dir.match(/[A-Z]/)) {
            return `Project/directory name cannot contain uppercase letters: ${directory}`;
          }

          return true;
        }
      });
      directory = newDir;
    }

    if (!directory) return; // make sure that project directory is empty

    const projectPath = _path.default.resolve(directory);

    if (_fs.default.existsSync(projectPath)) {
      const allowedFiles = ['.git', '.gitignore', '.devcontainer.json'];

      const conflictingFiles = _fs.default.readdirSync(projectPath).filter(function (f) {
        return !allowedFiles.includes(f);
      });

      if (conflictingFiles.length > 0) {
        console.log(`Directory ${_chalk.default.cyan(projectPath)} must be empty.`);
        return;
      }
    } else {
      _fs.default.mkdirSync(projectPath);
    } // allow short handle boilerplate (e.g. `node-basic`)


    if (boilerplate && !boilerplate.startsWith('http')) {
      const matchedBoilerplate = defaultBoilerplates.find(function (b) {
        return b.name === boilerplate;
      });

      if (matchedBoilerplate) {
        boilerplate = matchedBoilerplate.repo;
      } else {
        // allow shorthand GitHub URLs (e.g. `graphcool/graphcool-server-example`)
        boilerplate = getGitHubUrl(boilerplate);
      }
    } // interactive selection


    let installCommand;

    if (!boilerplate) {
      const maxNameLength = defaultBoilerplates.map(function (bp) {
        return bp.name.length;
      }).reduce(function (max, x) {
        return Math.max(max, x);
      }, 0);
      const choices = defaultBoilerplates.map(function (bp) {
        return `${(0, _lodash.padEnd)(bp.name, maxNameLength + 2)} ${bp.description}`;
      });
      const {
        choice
      } = yield context.prompt({
        type: 'list',
        name: 'choice',
        message: `Choose Maana boilerplate project:`,
        choices
      });
      const entry = defaultBoilerplates[choices.indexOf(choice)];
      boilerplate = entry.repo;
      installCommand = entry.installCommand;
    }

    if (!boilerplate) return; // download repo contents

    const zipInfo = getZipInfo(boilerplate);
    const downloadUrl = zipInfo.url;

    const tmpFile = _tmp.default.fileSync();

    console.log(`[mcreate] Downloading boilerplate from ${downloadUrl} to ${tmpFile.name}...`);
    yield new Promise(function (resolve) {
      (0, _request.default)(downloadUrl).pipe(_fs.default.createWriteStream(tmpFile.name)).on('close', resolve);
    });
    const zip = new _admZip.default(tmpFile.name); // Recent versions of adm-zip (without critical security vulnerability) take 'basename'
    // (i.e. file name only) for every entry if maintainEntryPath is set to false.
    // To work around this, after archive is extracted, move contents of extracted entry
    // to a level above and remove it

    zip.extractEntryTo(zipInfo.path, projectPath);

    const extractedPath = _path.default.join(projectPath, zipInfo.path);

    const dirents = _fs.default.readdirSync(extractedPath, {
      withFileTypes: true
    });

    dirents.forEach(function (entry) {
      _fs.default.renameSync(_path.default.join(extractedPath, entry.name), _path.default.join(projectPath, entry.name));
    });

    _rimraf.default.sync(extractedPath);

    tmpFile.removeCallback(); // run install command (if any)

    if (!noInstall && installCommand && installCommand !== '') {
      const subDirs = _fs.default.readdirSync(projectPath).map(function (f) {
        return _path.default.join(projectPath, f);
      }).filter(function (f) {
        return _fs.default.statSync(f).isDirectory();
      });

      const installPaths = [projectPath, ...subDirs].map(function (dir) {
        return _path.default.join(dir, 'package.json');
      }).filter(function (p) {
        return _fs.default.existsSync(p);
      });

      for (const packageJsonPath of installPaths) {
        process.chdir(_path.default.dirname(packageJsonPath));
        console.log(`[mcreate] Installing ${packageJsonPath}...`);

        if (_commandExists.default.sync(installCommand.split(' ')[0])) {
          yield shell(installCommand);
        } else {
          console.log(`Skipping install (no command: ${_chalk.default.cyan(installCommand)})`);
        }
      }
    } // change dir to projectPath for install steps


    process.chdir(projectPath); // run & delete setup script

    let installPath = _path.default.join(projectPath, 'install.js');

    if (!_fs.default.existsSync(installPath)) {
      installPath = _path.default.join(projectPath, '.install');
    }

    if (_fs.default.existsSync(installPath)) {
      console.log(`[mcreate] Running boilerplate install script... `);

      const installFunction = require(installPath);

      yield installFunction({
        context,
        project: _path.default.basename(projectPath),
        projectDir: directory
      });

      _rimraf.default.sync(installPath);
    }
  });

  return function handler(_x, _x2) {
    return _ref.apply(this, arguments);
  };
}();

exports.handler = handler;