"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.handler = exports.builder = exports.describe = exports.command = void 0;

var _chalk = _interopRequireDefault(require("chalk"));

var _graphqlConfig = require("graphql-config");

var _inquirer = _interopRequireDefault(require("inquirer"));

var _shelljs = _interopRequireDefault(require("shelljs"));

var _fs = _interopRequireDefault(require("fs"));

var _stripBom = _interopRequireDefault(require("strip-bom"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function asyncGeneratorStep(gen, resolve, reject, _next, _throw, key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { Promise.resolve(value).then(_next, _throw); } }

function _asyncToGenerator(fn) { return function () { var self = this, args = arguments; return new Promise(function (resolve, reject) { var gen = fn.apply(self, args); function _next(value) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "next", value); } function _throw(err) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "throw", err); } _next(undefined); }); }; }

var path = require('path');

const prompt = _inquirer.default.createPromptModule();

const scripts = {
  publish: __dirname + `/scripts/publish.sh`,
  deploy: __dirname + `/scripts/deploy.sh`,
  update: __dirname + `/scripts/update.sh`
}; // Total number of attempts mdeploy will try obtaining service ip address
// 10 * 12 = 120 seconds = 2 minutes

const maxRetries = 12;

function sleep(ms) {
  return new Promise(function (resolve) {
    return setTimeout(resolve, ms);
  });
}

const command = 'mdeploy'; // [serviceName] [servicePath] [registryPath] [versionTag] [numReplicas]

exports.command = command;
const describe = 'Deploy your service to Kubernetes';
exports.describe = describe;
const builder = {
  programatic: {
    alias: 'pr',
    describe: 'Disable the interactive mode of the CLI to use it programatically',
    type: 'boolean',
    default: false
  },
  serviceName: {
    alias: 'name',
    describe: 'The name for the service',
    type: 'string',
    default: ''
  },
  servicePath: {
    alias: 'path',
    describe: 'The path to the folder containing the Dockerfile for the service',
    type: 'string',
    default: ''
  },
  registryPath: {
    alias: 'registry',
    describe: 'The hostname to a container registry',
    type: 'string',
    default: ''
  },
  versionTag: {
    alias: 'tag',
    describe: 'The version tag for the service',
    type: 'string',
    default: ''
  },
  numReplicas: {
    alias: 'replicas',
    describe: 'The number of pods for the service',
    type: 'number',
    default: 0
  },
  port: {
    name: 'port',
    message: 'What is the port your application is running on?',
    default: 8050,
    type: 'input'
  }
};
exports.builder = builder;

const azureLogin = /*#__PURE__*/function () {
  var _ref = _asyncToGenerator(function* () {
    console.log(_chalk.default.blueBright('Please log in to your Azure account:'));
    const credentials = yield prompt([{
      message: 'Username:',
      type: 'input',
      name: 'username'
    }, {
      message: 'Password:',
      type: 'password',
      name: 'password'
    }]);
    const {
      username,
      password
    } = credentials;

    _shelljs.default.exec(`az login -u ${username} -p ${password}`);

    console.log(_chalk.default.green('Success!'));
  });

  return function azureLogin() {
    return _ref.apply(this, arguments);
  };
}();

const azureDeploy = /*#__PURE__*/function () {
  var _ref2 = _asyncToGenerator(function* () {
    const resourceGroupsResponse = _shelljs.default.exec('az group list -o json');

    const resourceGroups = JSON.parse(resourceGroupsResponse.stdout);
    const resourceGroupQuestion = [{
      message: 'Which resource group would you like to use?',
      name: 'resourceGroup',
      type: 'list',
      choices: resourceGroups
    }];
    const resourceGroupAnswer = yield prompt(resourceGroupQuestion);
    const {
      resourceGroup
    } = resourceGroupAnswer;

    const aksServicesRespinse = _shelljs.default.exec(`az aks list --resource-group ${resourceGroup}`);

    const aksServices = JSON.parse(aksServicesRespinse.stdout);
    const aksServiceQuestion = [{
      message: 'Which AKS cluster would you like to use?',
      name: 'aksService',
      type: 'list',
      choices: aksServices
    }];
    const aksServiceAnswer = yield prompt(aksServiceQuestion);
    const {
      aksService
    } = aksServiceAnswer;
    console.log(_chalk.default.blueBright('Getting AKS credentials'));

    _shelljs.default.exec(`az aks get-credentials --resource-group ${resourceGroup} --name ${aksService} --override`);

    console.log(_chalk.default.green('Authenticated successfully with AKS'));

    const spacesResponse = _shelljs.default.exec('azds space list -o json');

    const devSpacesNames = JSON.parse(spacesResponse.stdout).map(function (x) {
      return x.path;
    });
    const devSpaceAnswer = yield prompt([{
      message: 'Which Dev Space would you like to use?',
      name: 'devSpace',
      type: 'list',
      choices: devSpacesNames
    }]);
    const {
      devSpace
    } = devSpaceAnswer;
    console.log(_chalk.default.blueBright(`Selecting Dev Space `), _chalk.default.greenBright(devSpace));

    _shelljs.default.exec(`azds space select -n ${devSpace} -y`);

    console.log(_chalk.default.green('Dev space selected'));
    console.log(_chalk.default.blueBright(`Preparing`));

    _shelljs.default.exec('azds prep');

    console.log(_chalk.default.blueBright(`Deploying`));

    _shelljs.default.exec('azds up');

    console.log(_chalk.default.bgGreen(`Deployment complete`));
  });

  return function azureDeploy() {
    return _ref2.apply(this, arguments);
  };
}();

const printServiceAddresses = /*#__PURE__*/function () {
  var _ref3 = _asyncToGenerator(function* (serviceName, port, attempt = 0) {
    const result = _shelljs.default.exec(`kubectl get service "${serviceName}" -o 'jsonpath={.status.loadBalancer.ingress[].ip}'`, {
      silent: true
    });

    if (result.code != 0) {
      console.log(`${_chalk.default.red('Something went wrong, aborting')}`);
      process.exit(-1);
    }

    if (result.stdout === '') {
      if (attempt > maxRetries) {
        console.log(`${_chalk.default.red('Something went wrong, aborting')}`);
        process.exit(-1);
      } else {
        yield sleep(10000); // Sleep for 10 seconds before next attempt

        yield printServiceAddresses(serviceName, port, attempt + 1);
      }
    } else {
      console.log(`The external IP address for ${_chalk.default.blue(serviceName)} is\n\t${_chalk.default.green(result.stdout)}\n`);
      console.log(`The URL for your GraphQL endpoint is\n\t${_chalk.default.green('http://' + result.stdout + ':' + port + '/graphql')}\n`);
    }
  });

  return function printServiceAddresses(_x, _x2) {
    return _ref3.apply(this, arguments);
  };
}();

const buildAndPublishImage = /*#__PURE__*/function () {
  var _ref4 = _asyncToGenerator(function* (serviceName, servicePath, registryPath, versionTag) {
    const tag = `${registryPath}/${serviceName}:${versionTag}`;
    console.log(_chalk.default.green(`Building ${serviceName}`));

    let result = _shelljs.default.exec(`docker build ${servicePath} -t ${tag}`);

    if (result.code != 0) {
      console.log(`${_chalk.default.red('Something went wrong, aborting')}`);
      process.exit(-1);
    }

    console.log(_chalk.default.green(`Docker image build, publishing`));
    result = _shelljs.default.exec(`docker push ${tag}`);

    if (result.code != 0) {
      console.log(`${_chalk.default.red('Something went wrong, aborting')}`);
      process.exit(-1);
    }
  });

  return function buildAndPublishImage(_x3, _x4, _x5, _x6) {
    return _ref4.apply(this, arguments);
  };
}();

const registryDeploy = /*#__PURE__*/function () {
  var _ref5 = _asyncToGenerator(function* (serviceName, servicePath, registryPath, versionTag, numReplicas, port) {
    const templatePath = __dirname + '/scripts/deployment-service.yaml';

    const template = _fs.default.readFileSync(templatePath, {
      encoding: "utf8"
    });

    const manifest = template.replace(/\{\{SERVICE_NAME\}\}/g, serviceName).replace(/\{\{IMAGE\}\}/g, `${registryPath}/${serviceName}:${versionTag}`).replace(/\{\{PORT\}\}/g, port).replace(/\{\{REPLICAS\}\}/g, numReplicas);
    const manifestPath = `${servicePath}/${serviceName}.yaml`;

    _fs.default.writeFileSync(manifestPath, manifest, {
      encoding: 'utf8',
      flag: 'w'
    });

    const resolvedPath = _fs.default.realpathSync(manifestPath);

    console.log(`K8s deployment manifest file is saved in ${_chalk.default.green(resolvedPath)}.`);
    console.log('This file can be used to reproduce deployment on other K8s clusters by running:');
    console.log(_chalk.default.green(`\n\tkubectl apply -f ${resolvedPath}\n`));
    yield buildAndPublishImage(serviceName, servicePath, registryPath, versionTag);

    let result = _shelljs.default.exec(`kubectl apply -f ${manifestPath}`);

    if (result.code != 0) {
      console.log(`${_chalk.default.red('Something went wrong, aborting')}`);
      process.exit(-1);
    }

    printServiceAddresses(serviceName, port, 0);
  });

  return function registryDeploy(_x7, _x8, _x9, _x10, _x11, _x12) {
    return _ref5.apply(this, arguments);
  };
}();

const handler = /*#__PURE__*/function () {
  var _ref6 = _asyncToGenerator(function* (context, argv) {
    if (argv.programatic) {
      const {
        serviceName,
        servicePath,
        registryPath,
        versionTag,
        numReplicas,
        port
      } = argv;
      yield registryDeploy(serviceName, servicePath, registryPath, versionTag, numReplicas, port);
    } else {
      const questions = [{
        name: 'targetPlatform',
        message: 'What is target platform you are deplying to',
        type: 'list',
        choices: [{
          name: 'Private Docker Registry',
          value: 'registry'
        }, {
          name: 'Azure AKS (Must have Azure CLI installed)',
          value: 'aks'
        }]
      }];
      const answers = yield prompt(questions);

      switch (answers.targetPlatform) {
        case 'aks':
          const homedir = require('os').homedir();

          const credentials = yield _fs.default.readFileSync(homedir + '/.azure/azureProfile.json', 'utf8');
          const {
            subscriptions
          } = JSON.parse((0, _stripBom.default)(credentials));

          if (subscriptions.length === 0) {
            yield azureLogin();
          }

          yield azureDeploy();
          console.log(_chalk.default.green('Deployment on Azure AKS is Complete'));
          break;

        case 'registry':
          const serviceNameShell = path.basename(process.cwd());
          const registryQuestions = [{
            name: 'serviceName',
            message: 'What is the service name?',
            default: serviceNameShell,
            type: 'string'
          }, {
            name: 'servicePath',
            message: 'What is the path to the folder containing your Dockerfile?',
            default: process.cwd() + '/service',
            type: 'string'
          }, {
            name: 'registryPath',
            message: 'What is hostname for your container registry?',
            default: 'services.azurecr.io',
            type: 'string'
          }, {
            name: 'versionTag',
            message: 'What version tag you would like to use?',
            default: 'v1',
            type: 'string'
          }, {
            name: 'numReplicas',
            message: 'How many pods would you like to spin up?',
            default: 1,
            type: 'input'
          }, {
            name: 'port',
            message: 'What is the port your application is running on?',
            default: 8050,
            type: 'input'
          }];
          const registryOptions = yield prompt(registryQuestions);
          const {
            serviceName,
            servicePath,
            registryPath,
            versionTag,
            numReplicas,
            port
          } = registryOptions;
          const finalConfirmation = yield prompt({
            message: `Please confirm the following deployment plan:\n` + `Deploying the service ${_chalk.default.green(serviceName + ':' + versionTag)}\n` + `Located in ${_chalk.default.green(servicePath)}\n` + `Publishing to ${_chalk.default.green(registryPath)}\n` + `Number Of Pods: ${_chalk.default.green(numReplicas)}\n` + `Exposing port ${_chalk.default.green(port)}\n` + `Confirm?`,
            name: 'confirm',
            type: 'confirm'
          });

          if (finalConfirmation.confirm) {
            yield registryDeploy(serviceName, servicePath, registryPath, versionTag, numReplicas, port);
          } else {
            console.log('Exiting...');
          }

          break;
      }
    }
  });

  return function handler(_x13, _x14) {
    return _ref6.apply(this, arguments);
  };
}();

exports.handler = handler;