"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.handler = exports.builder = exports.desc = exports.command = void 0;

var _chalk = _interopRequireDefault(require("chalk"));

var _graphqlConfig = require("graphql-config");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function asyncGeneratorStep(gen, resolve, reject, _next, _throw, key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { Promise.resolve(value).then(_next, _throw); } }

function _asyncToGenerator(fn) { return function () { var self = this, args = arguments; return new Promise(function (resolve, reject) { var gen = fn.apply(self, args); function _next(value) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "next", value); } function _throw(err) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "throw", err); } _next(undefined); }); }; }

// Plugin boilerplate
const command = 'menv [--project] [--shell]';
exports.command = command;
const desc = 'Export the required environment variable(s)';
exports.desc = desc;
const builder = {
  shell: {
    alias: 's',
    description: 'The name of your current shell.  Defaults to pulling the name out of the the SHELL environment variable when available.  Examples: bash, cmd, or powershell'
  }
};
exports.builder = builder;

const handler = /*#__PURE__*/function () {
  var _ref = _asyncToGenerator(function* (context, argv) {
    // get the graphqlConfig
    let config = null;

    if (argv.project) {
      config = (0, _graphqlConfig.getGraphQLProjectConfig)(undefined, argv.project);
    } else {
      config = (0, _graphqlConfig.getGraphQLConfig)().config;
    }

    const extensions = config.extensions || {};
    const options = extensions.maana || {};

    if (!options.auth) {
      console.log(_chalk.default.red('✘ No authentication information available'));
      console.log(_chalk.default.yellow('Run'), _chalk.default.green('graphql msignin'), _chalk.default.yellow('to sign into the Maana CLI'));
      return;
    }

    const auth = JSON.parse(Buffer.from(options.auth, 'base64').toString());
    const shell = getShell(argv.shell);
    const outputConfig = getOutputConfig(shell, auth.access_token, argv.project);
    console.log(produceEvalOutput(outputConfig));
  });

  return function handler(_x, _x2) {
    return _ref.apply(this, arguments);
  };
}();

exports.handler = handler;

function produceEvalOutput(config) {
  return `${config.prefix}MAANA_AUTH_TOKEN${config.delimiter}${config.token}${config.suffix}${config.comment} Run this command to configure your shell\n${config.comment} ${config.cmd}`;
}

function getShell(shell) {
  if (!shell) {
    // TODO: Add additional checks for windows systems
    if (process.env.SHELL) {
      return process.env.SHELL.substr(process.env.SHELL.lastIndexOf('/') + 1);
    }
  }

  return shell;
}

function getOutputConfig(shell, token, project) {
  let shellConfig = {};
  let command = 'gql menv';
  if (project) command = `${command} --project ${project}`;
  if (shell) command = `${command} --shell ${shell}`;

  switch (shell) {
    case 'fish':
      shellConfig.prefix = 'set -gx ';
      shellConfig.suffix = '";\n';
      shellConfig.delimiter = ' "';
      shellConfig.comment = '#';
      shellConfig.cmd = `eval (${command})`;
      break;

    case 'powershell':
      shellConfig.prefix = '$Env:';
      shellConfig.suffix = '"\n';
      shellConfig.delimiter = ' = "';
      shellConfig.comment = '#';
      shellConfig.cmd = `& ${command} | Invoke-expression`;
      break;

    case 'cmd':
      shellConfig.prefix = 'SET ';
      shellConfig.suffix = '\n';
      shellConfig.delimiter = '=';
      shellConfig.comment = 'REM';
      shellConfig.cmd = `\t@FOR /f "tokens=*" %%i IN ('${command}') DO @%%i`;
      break;

    case 'tcsh':
      shellConfig.prefix = 'setenv ';
      shellConfig.suffix = '";\n';
      shellConfig.delimiter = ' "';
      shellConfig.comment = ':';
      shellConfig.cmd = `eval ${command}`;
      break;

    case 'emacs':
      shellConfig.prefix = '(setenv "';
      shellConfig.suffix = '")\n';
      shellConfig.delimiter = '" "';
      shellConfig.comment = ';;';
      shellConfig.cmd = `(with-temp-buffer (shell-command "${command}" (current-buffer)) (eval-buffer))`;
      break;

    default:
      shellConfig.prefix = 'export ';
      shellConfig.suffix = '"\n';
      shellConfig.delimiter = '="';
      shellConfig.comment = '#';
      shellConfig.cmd = `eval $(${command})`;
  }

  return { ...shellConfig,
    token
  };
}