"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.handler = exports.desc = exports.command = void 0;

var _chalk = _interopRequireDefault(require("chalk"));

var _requestPromiseNative = _interopRequireDefault(require("request-promise-native"));

var _graphqlConfig = require("graphql-config");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function asyncGeneratorStep(gen, resolve, reject, _next, _throw, key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { Promise.resolve(value).then(_next, _throw); } }

function _asyncToGenerator(fn) { return function () { var self = this, args = arguments; return new Promise(function (resolve, reject) { var gen = fn.apply(self, args); function _next(value) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "next", value); } function _throw(err) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "throw", err); } _next(undefined); }); }; }

var querystring = require('querystring'); // Plugin boilerplate


const command = 'mrefreshauth [--project]';
exports.command = command;
const desc = 'Refresh authentication of the Maana CLI';
exports.desc = desc;

const handler = /*#__PURE__*/function () {
  var _ref = _asyncToGenerator(function* (context, argv) {
    // get the full config file information
    let fullConfig = (0, _graphqlConfig.getGraphQLConfig)();
    let config = fullConfig.config; // if the user specified a specific project, we only want to edit the config
    // for that project

    if (argv.project) {
      if (config.projects && config.projects[argv.project]) {
        config = config.projects[argv.project];
      } else {
        console.log(_chalk.default.red(`✘ Failed to find the project ${argv.project}`));

        if (config.projects && Object.keys(config.projects).length > 0) {
          console.log(_chalk.default.yellow('Did you mean one of:'), _chalk.default.green(Object.keys(config.projects).join(', ')));
        } else {
          console.log(_chalk.default.yellow('Run'), _chalk.default.green('graphql add-project'), _chalk.default.yellow('to add a project to the .graphqlconfig'));
        }

        return;
      }
    }

    let extensions = config.extensions || {};
    if (!config.extensions) config.extensions = extensions;
    let maanaOptions = extensions.maana || {};
    if (!extensions.maana) extensions.maana = maanaOptions; // make sure we have the authentication config

    if (!maanaOptions.auth) {
      console.log(_chalk.default.red('✘ No authentication information available'));
      console.log(_chalk.default.yellow('Run'), _chalk.default.green('graphql msignin'), _chalk.default.yellow('to sign into the Maana CLI'));
      return;
    }

    let authConfig = JSON.parse(Buffer.from(maanaOptions.auth, 'base64').toString()); // This is a generic OAuth request and will
    // work for Auth0 or Keycloak.

    let form = {
      grant_type: 'refresh_token',
      client_id: authConfig.id,
      refresh_token: authConfig.refresh_token
    };
    var formData = querystring.stringify(form);
    var contentLength = formData.length;
    var requestConfig = {
      headers: {
        'Content-Length': contentLength,
        'Content-Type': 'application/x-www-form-urlencoded'
      },
      uri: authConfig.url,
      body: formData,
      method: 'POST'
    };

    try {
      let response = yield (0, _requestPromiseNative.default)(requestConfig); // build the auth information

      let authInfo = JSON.parse(response);
      authConfig.expires_at = Date.now() + authInfo.expires_in * 1000;
      authConfig.access_token = authInfo.access_token; // add auth information to the cofig and save it

      maanaOptions.auth = Buffer.from(JSON.stringify(authConfig)).toString('base64');
      fullConfig.saveConfig(fullConfig.config);
      console.log(_chalk.default.green("✔ Successfully refreshed Maana CLI's Auth"));
    } catch (e) {
      if (e.response && e.response.statusCode) {
        console.log(_chalk.default.red(`✘ Failed to refresh auth with status ${e.response.statusCode}`));

        if (e.response.body) {
          const body = JSON.parse(e.response.body);
          console.log(_chalk.default.yellow(`${body.error}: ${body.error_description}`));
        }
      } else {
        console.log(_chalk.default.red(`✘ Failed to refresh auth: ${e.message}`));
      }
    }
  });

  return function handler(_x, _x2) {
    return _ref.apply(this, arguments);
  };
}();

exports.handler = handler;