"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.handler = exports.desc = exports.command = void 0;

var _chalk = _interopRequireDefault(require("chalk"));

var _requestPromiseNative = _interopRequireDefault(require("request-promise-native"));

var _graphqlConfig = require("graphql-config");

var _util = require("./util");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function asyncGeneratorStep(gen, resolve, reject, _next, _throw, key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { Promise.resolve(value).then(_next, _throw); } }

function _asyncToGenerator(fn) { return function () { var self = this, args = arguments; return new Promise(function (resolve, reject) { var gen = fn.apply(self, args); function _next(value) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "next", value); } function _throw(err) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "throw", err); } _next(undefined); }); }; }

const querystring = require('querystring'); // Plugin boilerplate


const command = 'msignin [Authentication Token] [--project]';
exports.command = command;
const desc = 'Sign into the Maana CLI';
exports.desc = desc;

const handler = /*#__PURE__*/function () {
  var _ref = _asyncToGenerator(function* (context, argv) {
    // get the full config file information
    let fullConfig = (0, _graphqlConfig.getGraphQLConfig)();
    let config = fullConfig.config; // if the user specified a specific project, we only want to edit the config
    // for that project

    if (argv.project) {
      if (config.projects && config.projects[argv.project]) {
        config = config.projects[argv.project];
      } else {
        console.log(_chalk.default.red(`✘ Failed to find the project ${argv.project}`));

        if (config.projects && Object.keys(config.projects).length > 0) {
          console.log(_chalk.default.yellow('Did you mean one of:'), _chalk.default.green(Object.keys(config.projects).join(', ')));
        } else {
          console.log(_chalk.default.yellow('Run'), _chalk.default.green('graphql add-project'), _chalk.default.yellow('to add a project to the .graphqlconfig'));
        }

        return;
      }
    }

    let extensions = config.extensions || {};
    if (!config.extensions) config.extensions = extensions;
    let maanaOptions = extensions.maana || {};
    if (!extensions.maana) extensions.maana = maanaOptions; // make sure we have the authentication token

    let authConfig = null;

    if (argv.AuthenticationToken) {
      authConfig = JSON.parse(Buffer.from(argv.AuthenticationToken, 'base64').toString());
    } else {
      const {
        authToken
      } = yield context.prompt({
        type: 'password',
        name: 'authToken',
        message: 'Enter authentication token:'
      });

      if (!authToken) {
        console.log(_chalk.default.red('✘ Authentication token is required to sign into the Maana CLI'));
        return;
      }

      authConfig = JSON.parse(Buffer.from(authToken, 'base64').toString());
    }

    try {
      // Authorization code flow with PKCE
      // This is a generic OAuth request and will
      // work for Auth0 or Keycloak.
      var form = {
        grant_type: 'authorization_code',
        client_id: authConfig.id,
        code_verifier: Buffer.from(authConfig.state, 'base64').toString(),
        code: authConfig.code,
        redirect_uri: authConfig.ruri
      };
      var formData = querystring.stringify(form);
      var contentLength = formData.length;
      var requestConfig = {
        headers: {
          'Content-Length': contentLength,
          'Content-Type': 'application/x-www-form-urlencoded'
        },
        uri: authConfig.url,
        body: formData,
        method: 'POST'
      };
      let response = yield (0, _requestPromiseNative.default)(requestConfig);
      let authInfo = JSON.parse(response);
      authInfo.expires_at = Date.now() + authInfo.expires_in * 1000;
      authInfo.url = authConfig.url;
      authInfo.id = authConfig.id; // add auth information to the cofig and save it

      maanaOptions.auth = Buffer.from(JSON.stringify(authInfo)).toString('base64');
      (0, _util.addHeadersToConfig)(config);
      fullConfig.saveConfig(fullConfig.config);
      console.log(_chalk.default.green('✔ Successfully signed into Maana CLI')); // TODO:  Add a command that can be used to export the env variable
    } catch (e) {
      if (e.response && e.response.statusCode) {
        console.log(_chalk.default.red(`✘ Failed to sign in with status ${e.response.statusCode}`));

        if (e.response.body) {
          const body = JSON.parse(e.response.body);
          console.log(_chalk.default.yellow(`${body.error}: ${body.error_description}`));
        }
      } else {
        console.log(_chalk.default.red(`✘ Failed to sign in: ${e.message}`));
      }
    }
  });

  return function handler(_x, _x2) {
    return _ref.apply(this, arguments);
  };
}();

exports.handler = handler;